
//                                        Ruler
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890

    /********************************************************************/
    /*                                                                  */
    /*   The standard layout.                                           */
    /*                                                                  */
    /*   The standard layout for 'cpp' files in this code is as         */
    /*   follows:                                                       */
    /*                                                                  */
    /*      1. Include files.                                           */
    /*      2. Constants local to the class.                            */
    /*      3. Data structures local to the class.                      */
    /*      4. Data initializations.                                    */
    /*      5. Static functions.                                        */
    /*      6. Class functions.                                         */
    /*                                                                  */
    /*   The constructor is typically the first function, class         */
    /*   member functions appear in alphabetical order with the         */
    /*   destructor appearing at the end of the file.  Any section      */
    /*   or function this is not required is simply omitted.            */
    /*                                                                  */
    /********************************************************************/

#include "Global.hpp"

#include "DiskFile.hpp"
#include "Trigger.hpp"

    /********************************************************************/
    /*                                                                  */
    /*   A simple trigger.                                              */
    /*                                                                  */
    /*   A simple trigger is created for testing.                       */
    /*                                                                  */
    /********************************************************************/

class MY_TRIGGER : public TRIGGER
	{
		//
		//   Private data.
		//
		SBIT32						  Value;

	public:
		//
		//   Public functions.
		//
		MY_TRIGGER( SBIT32 NewValue )
			{ Value = NewValue; }

		VIRTUAL VOID First( TRIGGER_EVENT *TriggerEvent )
			{ 
			DebugPrint
				( 
				"Executing First(): %d,%d\n",
				TriggerEvent -> GetActual(),
				TriggerEvent -> GetOffset() 
				); 
			}

		VIRTUAL VOID Ascending( TRIGGER_EVENT *TriggerEvent )
			{ 
			DebugPrint
				( 
				"Executing Ascending(): %d,%d\n",
				TriggerEvent -> GetActual(),
				TriggerEvent -> GetOffset() 
				); 
			}

		VIRTUAL VOID Descending( TRIGGER_EVENT *TriggerEvent )
			{ 
			DebugPrint
				( 
				"Executing Descending(): %d,%d\n",
				TriggerEvent -> GetActual(),
				TriggerEvent -> GetOffset() 
				); 
			}

		VIRTUAL VOID Each( TRIGGER_EVENT *TriggerEvent )
			{ 
			DebugPrint
				( 
				"Executing Each(): %d,%d\n",
				TriggerEvent -> GetActual(),
				TriggerEvent -> GetOffset() 
				); 
			}

		VIRTUAL VOID Last( TRIGGER_EVENT *TriggerEvent )
			{ 
			DebugPrint
				( 
				"Executing Last(): %d,%d\n",
				TriggerEvent -> GetActual(),
				TriggerEvent -> GetOffset() 
				); 
			}
	};

    /********************************************************************/
    /*                                                                  */
    /*   Main program.                                                  */
    /*                                                                  */
    /*   The main program is the initial entry point for the system.    */
    /*                                                                  */
    /********************************************************************/

int _cdecl main( INT ArgC, CHAR *ArgV[] )
    {
	AUTO CHAR *WriteString = "Hello World\r\n";
	AUTO DISK_FILE DiskFile1( "C:\\Temp\\TestFile1.txt" );
	AUTO DISK_FILE DiskFile2( "C:\\Temp\\TestFile2.txt" );
	AUTO MY_TRIGGER *MyTrigger1 = new MY_TRIGGER( 4 );
	AUTO MY_TRIGGER *MyTrigger2 = new MY_TRIGGER( 4 );
	AUTO SBIT32 Count;

	printf( "Start of tests ...\n" );

	for( Count=0;Count < 10;Count ++ )
		{
		if ( !DiskFile1.Write( WriteString,strlen(WriteString),NoOffset,MyTrigger1 ) )
			{ Failure( "Write Fails !" ); }

		if ( !DiskFile2.Write( WriteString,strlen(WriteString),NoOffset,MyTrigger2 ) )
			{ Failure( "Write Fails !" ); }
		}

	MyTrigger1 -> Done();
	MyTrigger2 -> Done();

	printf( "End of tests ...\n" );

	return 0;
    }
